@@ -1,10 +1,12 @@
 #include "Popup.hpp"
 #include "../config/ConfigValue.hpp"
+#include "../config/ConfigManager.hpp"
 #include "../Compositor.hpp"
 #include "../protocols/LayerShell.hpp"
 #include "../protocols/XDGShell.hpp"
 #include "../protocols/core/Compositor.hpp"
 #include "../managers/SeatManager.hpp"
+#include "../managers/animation/AnimationManager.hpp"
 #include "../desktop/LayerSurface.hpp"
 #include "../managers/input/InputManager.hpp"
 #include "../render/Renderer.hpp"
@@ -51,6 +53,20 @@ CPopup::~CPopup() {
 
 void CPopup::initAllSignals() {
 
+    g_pAnimationManager->createAnimation(0.f, m_alpha, g_pConfigManager->getAnimationPropertyConfig("fadePopupsIn"), AVARDAMAGE_NONE);
+    m_alpha->setUpdateCallback([this](auto) {
+        //
+        g_pHyprRenderer->damageBox(CBox{coordsGlobal(), size()});
+    });
+    m_alpha->setCallbackOnEnd(
+        [this](auto) {
+            if (inert()) {
+                g_pHyprRenderer->damageBox(CBox{coordsGlobal(), size()});
+                fullyDestroy();
+            }
+        },
+        false);
+
     if (!m_resource) {
         if (!m_windowOwner.expired())
             m_listeners.newPopup = m_windowOwner->m_xdgSurface->m_events.newPopup.listen([this](const auto& resource) { this->onNewPopup(resource); });
@@ -74,7 +90,7 @@ void CPopup::initAllSignals() {
 void CPopup::onNewPopup(SP<CXDGPopupResource> popup) {
     const auto& POPUP = m_children.emplace_back(CPopup::create(popup, m_self));
     POPUP->m_self     = POPUP;
-    Debug::log(LOG, "New popup at {:x}", (uintptr_t)POPUP);
+    Debug::log(LOG, "New popup at {:x}", rc<uintptr_t>(this));
 }
 
 void CPopup::onDestroy() {
@@ -83,6 +99,24 @@ void CPopup::onDestroy() {
     if (!m_parent)
         return; // head node
 
+    m_subsurfaceHead.reset();
+    m_children.clear();
+    m_wlSurface.reset();
+
+    if (m_fadingOut && m_alpha->isBeingAnimated()) {
+        Debug::log(LOG, "popup {:x}: skipping full destroy, animating", rc<uintptr_t>(this));
+        return;
+    }
+
+    fullyDestroy();
+}
+
+void CPopup::fullyDestroy() {
+    Debug::log(LOG, "popup {:x} fully destroying", rc<uintptr_t>(this));
+
+    g_pHyprRenderer->makeEGLCurrent();
+    std::erase_if(g_pHyprOpenGL->m_popupFramebuffers, [&](const auto& other) { return other.first.expired() || other.first == m_self; });
+
     std::erase_if(m_parent->m_children, [this](const auto& other) { return other.get() == this; });
 }
 
@@ -112,6 +146,12 @@ void CPopup::onMap() {
 
     if (!m_layerOwner.expired() && m_layerOwner->m_layer < ZWLR_LAYER_SHELL_V1_LAYER_TOP)
         g_pHyprOpenGL->markBlurDirtyForMonitor(g_pCompositor->getMonitorFromID(m_layerOwner->m_layer));
+
+    m_alpha->setConfig(g_pConfigManager->getAnimationPropertyConfig("fadePopupsIn"));
+    m_alpha->setValueAndWarp(0.F);
+    *m_alpha = 1.F;
+
+    Debug::log(LOG, "popup {:x}: mapped", rc<uintptr_t>(this));
 }
 
 void CPopup::onUnmap() {
@@ -124,13 +164,14 @@ void CPopup::onUnmap() {
         return;
     }
 
-    m_mapped = false;
+    Debug::log(LOG, "popup {:x}: unmapped", rc<uintptr_t>(this));
 
     // if the popup committed a different size right now, we also need to damage the old size.
     const Vector2D MAX_DAMAGE_SIZE = {std::max(m_lastSize.x, m_resource->m_surface->m_surface->m_current.size.x),
                                       std::max(m_lastSize.y, m_resource->m_surface->m_surface->m_current.size.y)};
 
     m_lastSize = m_resource->m_surface->m_surface->m_current.size;
+    m_lastPos  = coordsRelativeToParent();
 
     const auto COORDS = coordsGlobal();
 
@@ -142,6 +183,17 @@ void CPopup::onUnmap() {
     box = CBox{COORDS, MAX_DAMAGE_SIZE}.expand(4);
     g_pHyprRenderer->damageBox(box);
 
+    m_lastSize = MAX_DAMAGE_SIZE;
+
+    g_pHyprRenderer->makeSnapshot(m_self);
+
+    m_fadingOut = true;
+    m_alpha->setConfig(g_pConfigManager->getAnimationPropertyConfig("fadePopupsOut"));
+    m_alpha->setValueAndWarp(1.F);
+    *m_alpha = 0.F;
+
+    m_mapped = false;
+
     m_subsurfaceHead.reset();
 
     if (!m_layerOwner.expired() && m_layerOwner->m_layer < ZWLR_LAYER_SHELL_V1_LAYER_TOP)
@@ -214,7 +266,7 @@ void CPopup::onCommit(bool ignoreSiblings) {
 }
 
 void CPopup::onReposition() {
-    Debug::log(LOG, "Popup {:x} requests reposition", (uintptr_t)this);
+    Debug::log(LOG, "Popup {:x} requests reposition", rc<uintptr_t>(this));
 
     m_requestedReposition = true;
 
@@ -245,7 +297,7 @@ Vector2D CPopup::coordsRelativeToParent() {
     Vector2D offset;
 
     if (!m_resource)
-        return {};
+        return m_lastPos;
 
     WP<CPopup> current = m_self;
     offset -= current->m_resource->m_surface->m_current.geometry.pos();
@@ -333,6 +385,9 @@ void CPopup::bfHelper(std::vector<WP<CPopup>> const& nodes, std::function<void(W
     nodes2.reserve(nodes.size() * 2);
 
     for (auto const& n : nodes) {
+        if (!n)
+            continue;
+
         for (auto const& c : n->m_children) {
             nodes2.push_back(c->m_self);
         }
@@ -343,6 +398,9 @@ void CPopup::bfHelper(std::vector<WP<CPopup>> const& nodes, std::function<void(W
 }
 
 void CPopup::breadthfirst(std::function<void(WP<CPopup>, void*)> fn, void* data) {
+    if (!m_self)
+        return;
+
     std::vector<WP<CPopup>> popups;
     popups.push_back(m_self);
     bfHelper(popups, fn, data);
@@ -381,3 +439,11 @@ WP<CPopup> CPopup::at(const Vector2D& globalCoords, bool allowsInput) {
 bool CPopup::inert() const {
     return m_inert;
 }
+
+PHLMONITOR CPopup::getMonitor() {
+    if (!m_windowOwner.expired())
+        return m_windowOwner->m_monitor.lock();
+    if (!m_layerOwner.expired())
+        return m_layerOwner->m_monitor.lock();
+    return nullptr;
+}
