@@ -15,12 +15,13 @@
 #include "../protocols/ToplevelExport.hpp"
 #include "../protocols/types/ContentType.hpp"
 #include "../xwayland/XSurface.hpp"
-#include "managers/AnimationManager.hpp"
+#include "managers/animation/AnimationManager.hpp"
+#include "managers/animation/DesktopAnimationManager.hpp"
 #include "managers/PointerManager.hpp"
 #include "../desktop/LayerSurface.hpp"
 #include "../managers/LayoutManager.hpp"
 #include "../managers/EventManager.hpp"
-#include "../managers/AnimationManager.hpp"
+#include "../managers/animation/AnimationManager.hpp"
 
 #include <hyprutils/string/String.hpp>
 using namespace Hyprutils::String;
@@ -41,7 +42,7 @@ static void setVector2DAnimToMove(WP<CBaseAnimatedVariable> pav) {
     if (!PAV)
         return;
 
-    CAnimatedVariable<Vector2D>* animvar = dynamic_cast<CAnimatedVariable<Vector2D>*>(PAV.get());
+    CAnimatedVariable<Vector2D>* animvar = dc<CAnimatedVariable<Vector2D>*>(PAV.get());
     animvar->setConfig(g_pConfigManager->getAnimationPropertyConfig("windowsMove"));
 
     const auto PHLWINDOW = animvar->m_Context.pWindow.lock();
@@ -50,7 +51,7 @@ static void setVector2DAnimToMove(WP<CBaseAnimatedVariable> pav) {
 }
 
 void Events::listener_mapWindow(void* owner, void* data) {
-    PHLWINDOW   PWINDOW = ((CWindow*)owner)->m_self.lock();
+    PHLWINDOW   PWINDOW = sc<CWindow*>(owner)->m_self.lock();
 
     static auto PINACTIVEALPHA     = CConfigValue<Hyprlang::FLOAT>("decoration:inactive_opacity");
     static auto PACTIVEALPHA       = CConfigValue<Hyprlang::FLOAT>("decoration:active_opacity");
@@ -181,7 +182,7 @@ void Events::listener_mapWindow(void* owner, void* data) {
                 break;
             }
             case CWindowRule::RULE_WORKSPACE: {
-                // check if it isnt unset
+                // check if it isn't unset
                 const auto WORKSPACERQ = r->m_rule.substr(r->m_rule.find_first_of(' ') + 1);
 
                 if (WORKSPACERQ == "unset")
@@ -223,7 +224,7 @@ void Events::listener_mapWindow(void* owner, void* data) {
                 try {
                     clientMode = std::stoi(ARGS[1]);
                 } catch (std::exception& e) { clientMode = 0; }
-                requestedFSState = SFullscreenState{.internal = (eFullscreenMode)internalMode, .client = (eFullscreenMode)clientMode};
+                requestedFSState = SFullscreenState{.internal = sc<eFullscreenMode>(internalMode), .client = sc<eFullscreenMode>(clientMode)};
                 break;
             }
             case CWindowRule::RULE_SUPPRESSEVENT: {
@@ -321,6 +322,7 @@ void Events::listener_mapWindow(void* owner, void* data) {
                 try {
                     PWINDOW->m_closeableSince = Time::steadyNow() + std::chrono::milliseconds(std::stoull(VARS[1]));
                 } catch (std::exception& e) { Debug::log(ERR, "Rule \"{}\" failed with: {}", r->m_rule, e.what()); }
+                break;
             }
             default: break;
         }
@@ -527,13 +529,13 @@ void Events::listener_mapWindow(void* owner, void* data) {
                         if (ONSCREEN) {
                             int borderSize = PWINDOW->getRealBorderSize();
 
-                            posX = std::clamp(posX, (int)(PMONITOR->m_reservedTopLeft.x + borderSize),
-                                              std::max((int)(PMONITOR->m_size.x - PMONITOR->m_reservedBottomRight.x - PWINDOW->m_realSize->goal().x - borderSize),
-                                                       (int)(PMONITOR->m_reservedTopLeft.x + borderSize + 1)));
+                            posX = std::clamp(posX, sc<int>(PMONITOR->m_reservedTopLeft.x + borderSize),
+                                              std::max(sc<int>(PMONITOR->m_size.x - PMONITOR->m_reservedBottomRight.x - PWINDOW->m_realSize->goal().x - borderSize),
+                                                       sc<int>(PMONITOR->m_reservedTopLeft.x + borderSize + 1)));
 
-                            posY = std::clamp(posY, (int)(PMONITOR->m_reservedTopLeft.y + borderSize),
-                                              std::max((int)(PMONITOR->m_size.y - PMONITOR->m_reservedBottomRight.y - PWINDOW->m_realSize->goal().y - borderSize),
-                                                       (int)(PMONITOR->m_reservedTopLeft.y + borderSize + 1)));
+                            posY = std::clamp(posY, sc<int>(PMONITOR->m_reservedTopLeft.y + borderSize),
+                                              std::max(sc<int>(PMONITOR->m_size.y - PMONITOR->m_reservedBottomRight.y - PWINDOW->m_realSize->goal().y - borderSize),
+                                                       sc<int>(PMONITOR->m_reservedTopLeft.y + borderSize + 1)));
                         }
 
                         Debug::log(LOG, "Rule move, applying to {}", PWINDOW);
@@ -631,9 +633,9 @@ void Events::listener_mapWindow(void* owner, void* data) {
     }
 
     if (requestedClientFSMode.has_value() && (PWINDOW->m_suppressedEvents & SUPPRESS_FULLSCREEN))
-        requestedClientFSMode = (eFullscreenMode)((uint8_t)requestedClientFSMode.value_or(FSMODE_NONE) & ~(uint8_t)FSMODE_FULLSCREEN);
+        requestedClientFSMode = sc<eFullscreenMode>(sc<uint8_t>(requestedClientFSMode.value_or(FSMODE_NONE)) & ~sc<uint8_t>(FSMODE_FULLSCREEN));
     if (requestedClientFSMode.has_value() && (PWINDOW->m_suppressedEvents & SUPPRESS_MAXIMIZE))
-        requestedClientFSMode = (eFullscreenMode)((uint8_t)requestedClientFSMode.value_or(FSMODE_NONE) & ~(uint8_t)FSMODE_MAXIMIZED);
+        requestedClientFSMode = sc<eFullscreenMode>(sc<uint8_t>(requestedClientFSMode.value_or(FSMODE_NONE)) & ~sc<uint8_t>(FSMODE_MAXIMIZED));
 
     if (!PWINDOW->m_noInitialFocus && (requestedInternalFSMode.has_value() || requestedClientFSMode.has_value() || requestedFSState.has_value())) {
         // fix fullscreen on requested (basically do a switcheroo)
@@ -687,9 +689,7 @@ void Events::listener_mapWindow(void* owner, void* data) {
     g_pLayoutManager->getCurrentLayout()->recalculateWindow(PWINDOW);
 
     // do animations
-    g_pAnimationManager->onWindowPostCreateClose(PWINDOW, false);
-    PWINDOW->m_alpha->setValueAndWarp(0.f);
-    *PWINDOW->m_alpha = 1.f;
+    g_pDesktopAnimationManager->startAnimation(PWINDOW, CDesktopAnimationManager::ANIMATION_TYPE_IN);
 
     PWINDOW->m_realPosition->setCallbackOnEnd(setVector2DAnimToMove);
     PWINDOW->m_realSize->setCallbackOnEnd(setVector2DAnimToMove);
@@ -717,7 +717,7 @@ void Events::listener_mapWindow(void* owner, void* data) {
 }
 
 void Events::listener_unmapWindow(void* owner, void* data) {
-    PHLWINDOW PWINDOW = ((CWindow*)owner)->m_self.lock();
+    PHLWINDOW PWINDOW = sc<CWindow*>(owner)->m_self.lock();
 
     Debug::log(LOG, "{:c} unmapped", PWINDOW);
 
@@ -826,7 +826,7 @@ void Events::listener_unmapWindow(void* owner, void* data) {
         if (PWINDOW == g_pCompositor->m_lastWindow.lock() || !g_pCompositor->m_lastWindow.lock()) {
             g_pEventManager->postEvent(SHyprIPCEvent{"activewindow", ","});
             g_pEventManager->postEvent(SHyprIPCEvent{"activewindowv2", ""});
-            EMIT_HOOK_EVENT("activeWindow", (PHLWINDOW) nullptr);
+            EMIT_HOOK_EVENT("activeWindow", PHLWINDOW{nullptr});
         }
     } else {
         Debug::log(LOG, "Unmapped was not focused, ignoring a refocus.");
@@ -837,11 +837,10 @@ void Events::listener_unmapWindow(void* owner, void* data) {
     g_pCompositor->addToFadingOutSafe(PWINDOW);
 
     if (!PWINDOW->m_X11DoesntWantBorders)                                                     // don't animate out if they weren't animated in.
-        *PWINDOW->m_realPosition = PWINDOW->m_realPosition->value() + Vector2D(0.01f, 0.01f); // it has to be animated, otherwise onWindowPostCreateClose will ignore it
+        *PWINDOW->m_realPosition = PWINDOW->m_realPosition->value() + Vector2D(0.01f, 0.01f); // it has to be animated, otherwise CesktopAnimationManager will ignore it
 
     // anims
-    g_pAnimationManager->onWindowPostCreateClose(PWINDOW, true);
-    *PWINDOW->m_alpha = 0.f;
+    g_pDesktopAnimationManager->startAnimation(PWINDOW, CDesktopAnimationManager::ANIMATION_TYPE_OUT);
 
     // recheck idle inhibitors
     g_pInputManager->recheckIdleInhibitorStatus();
@@ -855,7 +854,7 @@ void Events::listener_unmapWindow(void* owner, void* data) {
 }
 
 void Events::listener_commitWindow(void* owner, void* data) {
-    PHLWINDOW PWINDOW = ((CWindow*)owner)->m_self.lock();
+    PHLWINDOW PWINDOW = sc<CWindow*>(owner)->m_self.lock();
 
     if (!PWINDOW->m_isX11 && PWINDOW->m_xdgSurface->m_initialCommit) {
         Vector2D predSize = g_pLayoutManager->getCurrentLayout()->predictSizeForNewWindow(PWINDOW);
@@ -922,7 +921,7 @@ void Events::listener_commitWindow(void* owner, void* data) {
 }
 
 void Events::listener_destroyWindow(void* owner, void* data) {
-    PHLWINDOW PWINDOW = ((CWindow*)owner)->m_self.lock();
+    PHLWINDOW PWINDOW = sc<CWindow*>(owner)->m_self.lock();
 
     Debug::log(LOG, "{:c} destroyed, queueing.", PWINDOW);
 
@@ -953,7 +952,7 @@ void Events::listener_destroyWindow(void* owner, void* data) {
 }
 
 void Events::listener_activateX11(void* owner, void* data) {
-    PHLWINDOW PWINDOW = ((CWindow*)owner)->m_self.lock();
+    PHLWINDOW PWINDOW = sc<CWindow*>(owner)->m_self.lock();
 
     Debug::log(LOG, "X11 Activate request for window {}", PWINDOW);
 
@@ -978,7 +977,7 @@ void Events::listener_activateX11(void* owner, void* data) {
 }
 
 void Events::listener_unmanagedSetGeometry(void* owner, void* data) {
-    PHLWINDOW PWINDOW = ((CWindow*)owner)->m_self.lock();
+    PHLWINDOW PWINDOW = sc<CWindow*>(owner)->m_self.lock();
 
     if (!PWINDOW->m_isMapped || !PWINDOW->m_xwaylandSurface || !PWINDOW->m_xwaylandSurface->m_overrideRedirect)
         return;
