@@ -90,12 +90,14 @@ class CInputManager {
     void               onMouseMoved(IPointer::SMotionEvent);
     void               onMouseWarp(IPointer::SMotionAbsoluteEvent);
     void               onMouseButton(IPointer::SButtonEvent);
-    void               onMouseWheel(IPointer::SAxisEvent);
+    void               onMouseWheel(IPointer::SAxisEvent, SP<IPointer> pointer = nullptr);
     void               onKeyboardKey(const IKeyboard::SKeyEvent&, SP<IKeyboard>);
     void               onKeyboardMod(SP<IKeyboard>);
 
+    void               newKeyboard(SP<IKeyboard>);
     void               newKeyboard(SP<Aquamarine::IKeyboard>);
     void               newVirtualKeyboard(SP<CVirtualKeyboardV1Resource>);
+    void               newMouse(SP<IPointer>);
     void               newMouse(SP<Aquamarine::IPointer>);
     void               newVirtualMouse(SP<CVirtualPointerV1Resource>);
     void               newTouchDevice(SP<Aquamarine::ITouch>);
@@ -115,7 +117,7 @@ class CInputManager {
     bool               isLocked();
 
     Vector2D           getMouseCoordsInternal();
-    void               refocus();
+    void               refocus(std::optional<Vector2D> overridePos = std::nullopt);
     bool               refocusLastWindow(PHLMONITOR pMonitor);
     void               simulateMouseMovement();
     void               sendMotionEventsToFocused();
@@ -140,6 +142,10 @@ class CInputManager {
     void               onSwipeEnd(IPointer::SSwipeEndEvent);
     void               onSwipeUpdate(IPointer::SSwipeUpdateEvent);
 
+    void               onPinchBegin(IPointer::SPinchBeginEvent);
+    void               onPinchUpdate(IPointer::SPinchUpdateEvent);
+    void               onPinchEnd(IPointer::SPinchEndEvent);
+
     void               onTabletAxis(CTablet::SAxisEvent);
     void               onTabletProximity(CTablet::SProximityEvent);
     void               onTabletTip(CTablet::STipEvent);
@@ -178,14 +184,13 @@ class CInputManager {
     void              recheckIdleInhibitorStatus();
     bool              isWindowInhibiting(const PHLWINDOW& pWindow, bool onlyHl = true);
 
-    SSwipeGesture     m_activeSwipe;
-
     CTimer            m_lastCursorMovement;
 
     CInputMethodRelay m_relay;
 
     // for shared mods
-    uint32_t accumulateModsFromAllKBs();
+    const std::vector<uint32_t>& getKeysFromAllKBs();
+    uint32_t                     getModsFromAllKBs();
 
     // for virtual keyboards: whether we should respect them as normal ones
     bool shouldIgnoreVirtualKeyboard(SP<IKeyboard>);
@@ -194,7 +199,6 @@ class CInputManager {
     void        setCursorImageUntilUnset(std::string);
     void        unsetCursorImage();
 
-    std::string deviceNameToInternalString(std::string);
     std::string getNameForNewDevice(std::string);
 
     void        releaseAllMouseButtons();
@@ -243,7 +247,7 @@ class CInputManager {
 
     uint32_t           m_capabilities = 0;
 
-    void               mouseMoveUnified(uint32_t, bool refocus = false, bool mouse = false);
+    void               mouseMoveUnified(uint32_t, bool refocus = false, bool mouse = false, std::optional<Vector2D> overridePos = std::nullopt);
     void               recheckMouseWarpOnMouseInput();
 
     SP<CTabletTool>    ensureTabletToolPresent(SP<Aquamarine::ITabletTool>);
@@ -275,13 +279,8 @@ class CInputManager {
     };
     std::vector<UP<SIdleInhibitor>> m_idleInhibitors;
 
-    // swipe
-    void beginWorkspaceSwipe();
-    void updateWorkspaceSwipe(double);
-    void endWorkspaceSwipe();
-
-    void setBorderCursorIcon(eBorderIconDirection);
-    void setCursorIconOnBorder(PHLWINDOW w);
+    void                            setBorderCursorIcon(eBorderIconDirection);
+    void                            setCursorIconOnBorder(PHLWINDOW w);
 
     // temporary. Obeys setUntilUnset.
     void setCursorImageOverride(const std::string& name);
@@ -305,8 +304,14 @@ class CInputManager {
         uint32_t accumulatedScroll = 0;
     } m_scrollWheelState;
 
+    bool                  shareKeyFromAllKBs(uint32_t key, bool pressed);
+    uint32_t              shareModsFromAllKBs(uint32_t depressed);
+    std::vector<uint32_t> m_pressed;
+    uint32_t              m_lastMods = 0;
+
     friend class CKeybindManager;
     friend class CWLSurface;
+    friend class CWorkspaceSwipeGesture;
 };
 
 inline UP<CInputManager> g_pInputManager;
