@@ -7,11 +7,11 @@ CTextureMatteElement::CTextureMatteElement(const CTextureMatteElement::STextureM
 
 void CTextureMatteElement::draw(const CRegion& damage) {
     if (m_data.disableTransformAndModify) {
-        g_pHyprOpenGL->setMonitorTransformEnabled(true);
+        g_pHyprOpenGL->pushMonitorTransformEnabled(true);
         g_pHyprOpenGL->setRenderModifEnabled(false);
         g_pHyprOpenGL->renderTextureMatte(m_data.tex, m_data.box, *m_data.fb);
         g_pHyprOpenGL->setRenderModifEnabled(true);
-        g_pHyprOpenGL->setMonitorTransformEnabled(false);
+        g_pHyprOpenGL->popMonitorTransformEnabled();
     } else
         g_pHyprOpenGL->renderTextureMatte(m_data.tex, m_data.box, *m_data.fb);
 }
